Clojure only.

fogus.rv.productions.naive


noopclj

source

qfclj

(qf n)

Returns reducing function that terminates after n production firings.

Returns reducing function that terminates after n production firings.
sourceraw docstring

random-choiceclj

source

cljdoc builds & hosts documentation for Clojure/Script libraries