(qf n)Returns reducing function that terminates after n production firings.
Returns reducing function that terminates after n production firings.
cljdoc builds & hosts documentation for Clojure/Script libraries
| Ctrl+k | Jump to recent docs |
| ← | Move to previous article |
| → | Move to next article |
| Ctrl+/ | Jump to the search field |